﻿using Microsoft.Win32;
using System;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace DesktopAssist
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private static string GetHex(Color color)
        {
            return $"#{color.R:x2}{color.G:x2}{color.B:x2}";
        }

        private void SwatchButton_Click(object sender, EventArgs e)
        {
            var swatchButton = sender as Button;
            var selectedColor = swatchButton.BackColor;
            var selectedColorHex = GetHex(swatchButton.BackColor);

            SelectedSwatchPanel.BackColor = selectedColor;
            SelectedSwatchTextBox.Text = selectedColorHex;
        }

        private void CustomSwatchButton_Click(object sender, EventArgs e)
        {
            var colorDialog = new ColorDialog();

            if (colorDialog.ShowDialog() != DialogResult.OK)
                return;

            SelectedSwatchPanel.BackColor = colorDialog.Color;
            SelectedSwatchTextBox.Text = GetHex(colorDialog.Color);
        }

        private void ApplyButton_Click(object sender, EventArgs e)
        {
            WallpaperHelper.SetColor(SelectedSwatchPanel.BackColor);
            WallpaperHelper.DisableSlideshow();
        }
    }

    public static class WallpaperHelper
    {
        public static void SetColor(Color color)
        {
            // Remove the current wallpaper
            NativeMethods.SystemParametersInfo(
                NativeMethods.SPI_SETDESKWALLPAPER, 0, "",
                NativeMethods.SPIF_UPDATEINIFILE | NativeMethods.SPIF_SENDWININICHANGE);

            // Set the new desktop solid color for the current session
            var elements = new int[] { NativeMethods.COLOR_DESKTOP };
            var colors = new int[] { ColorTranslator.ToWin32(color) };
            NativeMethods.SetSysColors(elements.Length, elements, colors);

            // Save value in registry so that it will persist
            var key = Registry.CurrentUser.OpenSubKey("Control Panel\\Colors", true);
            key.SetValue(@"Background", string.Format("{0} {1} {2}", color.R, color.G, color.B));
        }

        public static void DisableSlideshow()
        {
            try
            {
                const string RegistryPath = "Control Panel\\Personalization";
                const string SubKeyName = "Desktop Slideshow";

                var key = Registry.CurrentUser.OpenSubKey(RegistryPath, true);

                if (key is null)
                    return;

                if (key.GetSubKeyNames().Contains(SubKeyName))
                    key.DeleteSubKey(SubKeyName);
            }
            catch
            {
                // ignored
            }

        }
    }

    public static class NativeMethods
    {
        public const int COLOR_DESKTOP = 1;
        public const int SPI_SETDESKWALLPAPER = 20;
        public const int SPIF_UPDATEINIFILE = 0x01;
        public const int SPIF_SENDWININICHANGE = 0x02;

        [DllImport("user32.dll")]
        public static extern bool SetSysColors(int cElements, int[] lpaElements, int[] lpaRgbValues);

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern int SystemParametersInfo(int uAction, int uParam, string lpvParam, int fuWinIni);
    }
}
